"""
tests -- the actual TestCase (just one)
"""

import sys
import os
import unittest
import time

from . import util
from .util import Global
from .iscsi import IscsiData


def s2dt(s):
    """
    seconds to "HH:MM:SS.sss"
    """
    s_orig = s
    hrs = s / 3600
    s -= (int(hrs) * 3600)
    mins = s / 60
    s -= (int(mins) * 60)
    a_str="%02d:%02d:%06.3f" % (hrs, mins, s)
    util.dprint("s2dt: %f -> %s" % (s_orig, a_str))
    return a_str


def print_time_values():
    """
    print out global exec time values
    """
    util.vprint('')
    util.vprint('Times spent running sub-programs:')
    ttl_time = 0.0
    for s in Global.timing.keys():
        v = Global.timing[s]
        r = s2dt(v)
        ttl_time += v
        util.vprint('  %10s = %s' % (s, r))
    util.vprint('    =======================')
    util.vprint('  %10s = %s' % ('Total', s2dt(ttl_time)))
    util.vprint('')
    util.vprint('Total test-run time: %s' % (s2dt(Global.total_time)))

def verify_node_in_db():
    """
    Verify that the required IQN has a node in the database

    return (result, reason) (reason only matters if result is false)
    """
    util.dprint('verifying that Node is in DB: %s at %s' % 
                (Global.target, Global.ipnr))
    # check that our target node exists in the Node DB
    res = util.run_cmd(['iscsiadm', '-m', 'node',
                        '-T', Global.target,
                        '-p', Global.ipnr])
    if res == 0:
        return (res, 'success')
    util.dprint('Node not in the DB ... running discovery')
    # we need to run discovery, then verify the node is in the DB
    res = util.run_cmd(['iscsiadm', '-m', 'discovery', '-t', 'st',
                        '-p', Global.ipnr])
    if res != 0:
        return (res, 'Unable to run discovery')
    # verify node entry got created by our discovery
    res = util.run_cmd(['iscsiadm', '-m', 'node',
                        '-T', Global.target,
                        '-p', Global.ipnr])
    if res == 0:
        util.dprint('Success: node now in DB!')
        return (res, 'success')
    return (res, 'Discovery failed to create Node DB entry')

class TestRegression(unittest.TestCase):
    """
    Regression testing
    """
    @classmethod
    def setUpClass(cls):
        util.verify_needed_commands_exist(['parted', 'fio', Global.MKFSCMD[0], 'bonnie++', 'sgdisk', 'iscsiadm'])
        util.vprint('*** Starting %s' % cls.__name__)
        # an array of first burts, max burst, and max recv values, for testing
        cls.param_values = [[4096, 4096, 4096],
                            [8192, 4096, 4096],
                            [16384, 4096, 4096],
                            [32768, 4096, 4096],
                            [65536, 4096, 4096],
                            [131972, 4096, 4096],
                            [4096, 8192, 4096],
                            [4096, 16384, 4096],
                            [4096, 32768, 4096],
                            [4096, 65536, 4096],
                            [4096, 131072, 4096],
                            [4096, 4096, 8192],
                            [4096, 4096, 16384],
                            [4096, 4096, 32768],
                            [4096, 4096, 65536],
                            [4096, 4096, 131072]]
        cls.time_start = time.perf_counter()

    def setUp(self):
        if Global.debug or Global.verbosity > 1:
            # this makes debug printing a little more clean
            print('', file=sys.stderr)
        # validate that target exists and we have a node in the DB for it
        (res, reason) = verify_node_in_db()
        self.assertEqual(res, 0, reason)
        return True

    def iscsi_logout(self):
        res = util.run_cmd(['iscsiadm', '-m', 'node',
                            '-T', Global.target,
                            '-p', Global.ipnr,
                            '--logout'])
        if res not in [0, 21]:
            self.fail('logout failed')
        self.assertFalse(os.path.exists(Global.device), '%s: exists after logout!' % Global.device)

    def test_InitialR2T_on_ImmediateData_off(self):
        """Test Initial Request to Transmit on, but Immediate Data off"""
        i = 1
        for v in self.param_values:
            with self.subTest('Testing FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v), i=i):
                if i not in Global.subtest_list:
                    util.vprint('Skipping subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                else:
                    util.vprint('Running subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                    self.iscsi_logout()
                    iscsi_data = IscsiData('No', 'Yes', 'None', 'None', v[0], v[1], v[2])
                    iscsi_data.update_cfg(Global.target, Global.ipnr)
                    self.run_the_rest()
            i += 1

    def test_InitialR2T_off_ImmediateData_on(self):
        """Test Initial Request to Transmit off, Immediate Data on"""
        i = 1
        for v in self.param_values:
            with self.subTest('Testing FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v), i=i):
                if i not in Global.subtest_list:
                    util.vprint('Skipping subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                else:
                    util.vprint('Running subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                    self.iscsi_logout()
                    iscsi_data = IscsiData('Yes', 'No', 'None', 'None', v[0], v[1], v[2])
                    iscsi_data.update_cfg(Global.target, Global.ipnr)
                    self.run_the_rest()
            i += 1

    def test_InitialR2T_on_ImmediateData_on(self):
        """Test Initial Request to Transmit and Immediate Data on"""
        i = 1
        for v in self.param_values:
            with self.subTest('Testing FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v), i=i):
                if i not in Global.subtest_list:
                    util.vprint('Skipping subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                else:
                    util.vprint('Running subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                    self.iscsi_logout()
                    iscsi_data = IscsiData('Yes', 'Yes', 'None', 'None', v[0], v[1], v[2])
                    iscsi_data.update_cfg(Global.target, Global.ipnr)
                    self.run_the_rest()
            i += 1

    def test_InitialR2T_off_ImmediateData_off(self):
        """Test Initial Request to Transmit and Immediate Data off"""
        i = 1
        for v in self.param_values:
            with self.subTest('Testing FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v), i=i):
                if i not in Global.subtest_list:
                    util.vprint('Skipping subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                else:
                    util.vprint('Running subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                    self.iscsi_logout()
                    iscsi_data = IscsiData('No', 'No', 'None', 'None', v[0], v[1], v[2])
                    iscsi_data.update_cfg(Global.target, Global.ipnr)
                    self.run_the_rest()
            i += 1

    def test_HdrDigest_on_DataDigest_off(self):
        """Test With Header Digest"""
        i = 1
        for v in self.param_values:
            with self.subTest('Testing FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v), i=i):
                if i not in Global.subtest_list:
                    util.vprint('Skipping subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                else:
                    util.vprint('Running subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                    self.iscsi_logout()
                    iscsi_data = IscsiData('No', 'Yes', 'CRC32C', 'None', v[0], v[1], v[2])
                    iscsi_data.update_cfg(Global.target, Global.ipnr)
                    self.run_the_rest()
            i += 1

    def test_HdrDigest_on_DataDigest_on(self):
        """Test With Header Digest"""
        i = 1
        for v in self.param_values:
            with self.subTest('Testing FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v), i=i):
                if i not in Global.subtest_list:
                    util.vprint('Skipping subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                else:
                    util.vprint('Running subtest %d: FirstBurst={} MaxBurst={} MaxRecv={}'.format(*v) % i)
                    self.iscsi_logout()
                    iscsi_data = IscsiData('No', 'Yes', 'CRC32C', 'CRC32C', v[0], v[1], v[2])
                    iscsi_data.update_cfg(Global.target, Global.ipnr)
                    self.run_the_rest()
            i += 1

    def run_the_rest(self):
        res = util.run_cmd(['iscsiadm', '-m', 'node',
                            '-T', Global.target,
                            '-p', Global.ipnr,
                            '--login'])
        self.assertEqual(res, 0, 'cannot login to device')
        # wait a few seconds for the device to show up
        if not util.wait_for_path(Global.device):
            self.fail('%s: does not exist after login' % Global.device)
        # run parted to partition the disc with one whole disk partition
        (res, reason) = util.run_parted()
        self.assertEqual(res, 0, reason)
        # run fio to test file IO
        (res, reason) = util.run_fio()
        self.assertEqual(res, 0, reason)
        # wait a bit for cache to flush
        util.sleep_some(1)
        # make a filesystem
        (res, reason) = util.run_mkfs()
        self.assertEqual(res, 0, reason)
        # run bonnie++ to test the filesystem IO
        (res, reason) = util.run_bonnie()
        self.assertEqual(res, 0, reason)

    @classmethod
    def tearDownClass(cls):
        # restore iscsi config
        iscsi_data = IscsiData()
        iscsi_data.update_cfg(Global.target, Global.ipnr)
        # log out of iscsi connection
        util.run_cmd(['iscsiadm', '-m', 'node',
                      '-T', Global.target,
                      '-p', Global.ipnr,
                      '--logout'])
        Global.total_time = time.perf_counter() - cls.time_start
        print_time_values()



"""
harness stuff (support) -- utility routines
"""

import os
import shutil
import sys
import unittest
import time
import tempfile
import re

from . import __version__ as lib_version

#
# globals
#
class Global:
    FSTYPE = os.getenv('FSTYPE', 'ext3')
    if os.getenv('MOUNTOPTIONS'):
        MOUNTOPTIONS = os.getenv('MOUNTOPTIONS').split(' ')
    else:
        MOUNTOPTIONS = []
    MOUNTOPTIONS += ['-t', FSTYPE]
    MKFSCMD = [os.getenv('MKFSCMD', 'mkfs.' + FSTYPE)]
    if os.getenv('MKFSOPTS'):
        MKFSCMD += os.getenv('MKFSOPTS').split(' ')
    BONNIEPARAMS = os.getenv('BONNIEPARAMS', '-r0 -n10:0:0 -s16 -uroot -f -q').split(' ')
    verbosity = 1
    debug = False
    # the target (e.g. "iqn.*")
    target = None
    # the IP and optional port (e.g. "linux-system", "192.168.10.1:3260")
    ipnr = None
    # the device that will be created when our target is connected
    device = None
    # the first and only partition on said device
    partition = None
    # optional override for fio disk testing block size(s)
    blocksize = None
    # subtests to run -- by default, all of them
    # XXX we should really look how many subtests there are, but there's
    # no good way to detect that.
    subtest_list = [i+1 for i in range(16)]
    # for timing
    timing = dict.fromkeys(['fio', 'sgdisk', 'dd', 'bonnie', 'mkfs', 'sleep'], 0.0)
    total_time = 0.0


def dprint(*args):
    """
    Print a debug message if in debug mode
    """
    if Global.debug:
        print('DEBUG: ', file=sys.stderr, end='')
        for arg in args:
            print(arg, file=sys.stderr, end='')
        print('', file=sys.stderr)

def vprint(*args):
    """
    Print a verbose message
    """
    if Global.verbosity > 1 and args:
        for arg in args:
            print(arg, end='')
        print('')

def run_cmd(cmd, output_save_file=None):
    """
    run specified command, waiting for and returning result
    """
    if Global.debug:
        cmd_str = ' '.join(cmd)
        if output_save_file:
            cmd_str += ' >& %s' % output_save_file
        dprint(cmd_str)
    pid = os.fork()
    if pid < 0:
        print("Error: cannot fork!", flie=sys.stderr)
        sys.exit(1)
    if pid == 0:
        # the child
        if output_save_file or not Global.debug:
            stdout_fileno = sys.stdout.fileno()
            stderr_fileno = sys.stderr.fileno()
            if output_save_file:
                new_stdout = os.open(output_save_file, os.O_WRONLY|os.O_CREAT|os.O_TRUNC,
                                     mode=0o664)
            else:
                new_stdout = os.open('/dev/null', os.O_WRONLY)
            os.dup2(new_stdout, stdout_fileno)
            os.dup2(new_stdout, stderr_fileno)
        os.execvp(cmd[0], cmd)
        # not reached
        sys.exit(1)

    # the parent
    wpid, wstat = os.waitpid(pid, 0)
    if wstat != 0:
        dprint("exit status: (%d) %d" % (wstat, os.WEXITSTATUS(wstat)))
    return os.WEXITSTATUS(wstat)

def new_initArgParsers(self):
    """
    Add  some options to the normal unittest main options
    """
    global old_initArgParsers

    old_initArgParsers(self)
    self._main_parser.add_argument('-d', '--debug', dest='debug',
            action='store_true',
            help='Enable developer debugging')
    self._main_parser.add_argument('-t', '--target', dest='target',
            action='store',
            help='Required: target name')
    self._main_parser.add_argument('-i', '--ipnr', dest='ipnr',
            action='store',
            help='Required: name-or-ip[:port]')
    self._main_parser.add_argument('-D', '--device', dest='device',
            action='store',
            help='Required: device')
    self._main_parser.add_argument('-B', '--blocksize', dest='blocksize',
            action='store',
            help='block size (defaults to an assortment of sizes)')
    self._main_parser.add_argument('-V', '--version', dest='version_request',
            action='store_true',
            help='Display Version info and exit')
    self._main_parser.add_argument('-l', '--list', dest='list_tests',
            action='store_true',
            help='List test cases and exit')
    self._main_parser.add_argument('-s', '--subtests', dest='subtest_list',
            action='store',
            help='Subtests to execute [default all, i.e. "1-16"]')

def print_suite(suite):
    """Print a list of tests from a test suite"""
    dprint("print_suite: entering for", suite)
    if hasattr(suite, '__iter__'):
        for x in suite:
            print_suite(x)
    else:
        print(suite)

def new_parseArgs(self, argv):
    """
    Gather globals from unittest main for local consumption -- this
    called to parse then validate the arguments, inside each TestCase
    instance.
    """
    global old_parseArgs, prog_name, parent_version, lib_version

    # actually parse the arguments
    old_parseArgs(self, argv)

    # now validate stuff
    if self.version_request:
        print('%s Version %s, harness version %s' % \
              (prog_name, parent_version, lib_version))
        sys.exit(0)
    Global.verbosity = self.verbosity
    Global.debug = self.debug
    if self.list_tests:
        print_suite(unittest.defaultTestLoader.discover('.'))
        sys.exit(0)
    for v in ['target', 'ipnr', 'device']:
        if getattr(self, v) is None:
            print('Error: "%s" required' % v.upper())
            sys.exit(1)
        setattr(Global, v, getattr(self, v))
    Global.blocksize = self.blocksize
    dprint("found: verbosity=%d, target=%s, ipnr=%s, device=%s, bs=%s" % \
            (Global.verbosity, Global.target, Global.ipnr, Global.device, Global.blocksize))
    # get partition from path
    device_dir = os.path.dirname(Global.device)
    if device_dir == '/dev':
        Global.partition = '%s1' % Global.device
    elif device_dir in ['/dev/disk/by-id', '/dev/disk/by-path']:
        Global.partition = '%s-part1' % Global.device
    else:
        print('Error: must start with "/dev" or "/dev/disk/by-{id,path}": %s' % \
                Global.device, file=sys.stderr)
        sys.exit(1)
    if self.subtest_list:
        if not user_spec_to_list(self.subtest_list):
            self._print_help()
            sys.exit(1)

def user_spec_to_list(user_spec):
    """
    We have 16 subtests. By default, we run them all, but if
    the user has specified a subset, like 'N' or 'N-M', then
    a list of the indicies they requested.

    XXX: expand to handle groups, e.g. 1,3-4,12 ???

    XXX: should we validate that the range will work, or just
    let an exception happen in that case?
    """
    pat_single = re.compile(r'(\d+)$')
    pat_range = re.compile(r'(\d+)-(\d+)$')
    found = False
    start_idx = None
    end_idx = None
    res = pat_range.match(user_spec)
    if res:
        # user wants just one subtest
        start_idx = int(res.group(1)) - 1
        end_idx = int(res.group(2))
        dprint("Found request for range: %d-%d" % (start_idx, end_idx))
        found = True
    else:
        res = pat_single.match(user_spec)
        if res:
            start_idx = int(res.group(1)) - 1
            end_idx = start_idx + 1
            dprint("Found request for single: %d-%d" % (start_idx, end_idx))
            found = True
    if not found:
        print('Error: subtest spec does not match N or N-M: %s' % user_spec)
    else:
        dprint("subtest_list before:", Global.subtest_list)
        Global.subtest_list = Global.subtest_list[start_idx:end_idx]
        dprint("subtest_list after:", Global.subtest_list)
    return found
    
def setup_testProgram_overrides(version_str, name):
    """
    Add in special handling for a couple of the methods in TestProgram (main)
    so that we can add parameters and detect some globals we care about
    """
    global old_parseArgs, old_initArgParsers, parent_version, prog_name

    old_initArgParsers = unittest.TestProgram._initArgParsers
    unittest.TestProgram._initArgParsers = new_initArgParsers
    old_parseArgs = unittest.TestProgram.parseArgs
    unittest.TestProgram.parseArgs = new_parseArgs
    parent_version = version_str
    prog_name = name

def verify_needed_commands_exist(cmd_list):
    """
    Verify that the commands in the supplied list are in our path
    """
    path_list = os.getenv('PATH').split(':')
    any_cmd_not_found = False
    for cmd in cmd_list:
        found = False
        for a_path in path_list:
            if os.path.exists('%s/%s' % (a_path, cmd)):
                found = True
                break
        if not found:
            print('Error: %s must be in your PATH' % cmd)
            any_cmd_not_found = True
    if any_cmd_not_found:
        sys.exit(1)


def run_fio():
    """
    Run the fio benchmark for various block sizes.
    
    Return zero for success.
    Return non-zero for failure and a failure reason.

    Uses Globals: partition, blocksize
    """
    if Global.blocksize is not None:
        dprint('Found a block size passed in: %s' % Global.blocksize)
        blocksizes = Global.blocksize.split(' ')
    else:
        dprint('NO Global block size pass in?')
        blocksizes = ['512', '1k', '2k', '4k', '8k',
                '16k', '32k', '75536', '128k', '1000000']
    # for each block size, do a read test, then a write test
    for bs in blocksizes:
        vprint('Running "fio" read/write/verify tests with 8/8/1 threads, bs=%s' % bs)
        # only support direct IO with aligned reads
        if bs.endswith('k'):
            direct=1
        else:
            direct=0
        ts = time.perf_counter()
        res = run_cmd(['fio', '--name=read-test', '--readwrite=randread',
            '--runtime=2s', '--numjobs=8', '--blocksize=%s' % bs, 
            '--offset=256k',
            '--direct=%d' % direct, '--filename=%s' % Global.partition])
        if res != 0:
            return (res, 'fio failed')
        res = run_cmd(['fio', '--name=write-test', '--readwrite=randwrite',
            '--runtime=2s', '--numjobs=8', '--blocksize=%s' % bs, 
            '--offset=256k',
            '--direct=%d' % direct, '--filename=%s' % Global.partition])
        if res != 0:
            return (res, 'fio failed')
        res = run_cmd(['fio', '--name=verify-test', '--readwrite=randwrite',
            '--runtime=2s', '--numjobs=1', '--blocksize=%s' % bs, 
            '--direct=%d' % direct, '--filename=%s' % Global.partition,
            '--offset=256k',
            '--verify=md5', '--verify_state_save=0'])
        if res != 0:
            return (res, 'fio failed')
        te = time.perf_counter()
        Global.timing['fio'] += te - ts
    return (0, 'Success')

def wait_for_path(path, present=True, amt=10):
    """
    Wait until a path exists or is gone
    """
    dprint("Looking for path=%s, present=%s" % (path, present))
    for i in range(amt):
        sleep_some(1)
        if os.path.exists(path) == present:
            dprint("We are Happy :) present=%s, cnt=%d" % (present, i))
            return True
    dprint("We are not happy :( present=%s actual=%s after %d seconds" % \
           (present, os.path.exists(path), amt))
    return False

def wipe_disc():
    """
    Wipe the label and partition table from the disc drive -- the sleep-s
    are needed to give the async OS and udev a chance to notice the partition
    table has been erased
    """
    # zero out the label and parition table
    vprint('Running "sgdisk" and "dd" to wipe disc label, partitions, and filesystem')
    sleep_some(1)
    ts = time.perf_counter()
    res = run_cmd(['sgdisk', '--clear', Global.device])
    te = time.perf_counter()
    Global.timing['sgdisk'] += te -ts
    if res != 0:
        if res == 4:
            dprint("Oh oh -- 'clear' failed! trying one more time ...")
        ts = time.perf_counter()
        res = run_cmd(['sgdisk', '--clear', Global.device])
        te = time.perf_counter()
        if res != 0:
            return (res, '%s: could not zero out label after two tries: %d' % \
                    (Global.device, res))
        Global.timing['sgdisk'] += te - ts
    ts = time.perf_counter()
    res = run_cmd(['dd', 'if=/dev/zero', 'of=%s' % Global.device, 'bs=256k', 'count=20', 'oflag=direct'])
    te = time.perf_counter()
    if res != 0:
        return (res, '%s: could not zero out filesystem: %d' % (Global.device, res))
    Global.timing['dd'] += te - ts
    return (0, 'Success')
    
def run_parted():
    """
    Run the parted program to ensure there is one partition,
    and that it covers the whole disk

    Return zero for success and the device pathname.
    Return non-zero for failure and a failure reason.

    Uses Globals: device, partition
    """
    (res, reason) = wipe_disc()
    if res != 0:
        return (res, reason)
    # ensure our partition file is not there, to be safe
    if not wait_for_path(Global.partition, present=False, amt=30):
        return (1, '%s: Partition already exists?' % Global.partition)
    # make a label, then a partition table with one partition
    vprint('Running "parted" to create a label and partition table')
    res = run_cmd(['parted', Global.device, 'mklabel', 'gpt'])
    if res != 0:
        return (res, '%s: Could not create a GPT label' % Global.device)
    res = run_cmd(['parted', '-a', 'none', Global.device, 'mkpart', 'primary', '0', '100%'])
    if res != 0:
        return (res, '%s: Could not create a primary partition' % Global.device)
    # wait for the partition to show up
    if not wait_for_path(Global.partition):
        return (1, '%s: Partition never showed up?' % Global.partition)
    # success
    return (0, 'Success')

def run_mkfs():
    vprint('Running "mkfs" to to create filesystem')
    ts = time.perf_counter()
    res = run_cmd(Global.MKFSCMD + [ Global.partition ] )
    te = time.perf_counter()
    if res != 0:
        return (res, '%s: mkfs failed (%d)' % (Global.partition, res))
    Global.timing['mkfs'] += te - ts
    return (0, 'Success')

def run_bonnie():
    # make a temp dir and mount the device there
    with tempfile.TemporaryDirectory() as tmp_dir:
        vprint('Running "mount" to mount the filesystem')
        res = run_cmd(['mount'] + Global.MOUNTOPTIONS + [Global.partition, tmp_dir])
        if res != 0:
            return (res, '%s: mount failed (%d)' % (Global.partition, res))
        # run bonnie++ on the new directory
        vprint('Running "bonnie++" on the filesystem')
        ts = time.perf_counter()
        res = run_cmd(['bonnie++'] + Global.BONNIEPARAMS + ['-d', tmp_dir])
        te = time.perf_counter()
        if res != 0:
            return (res, '%s: umount failed (%d)' % (tmp_dir, res))
        Global.timing['bonnie'] += te - ts
        # unmount the device and remove the temp dir
        vprint('Running "umount" to unmount the filesystem')
        res = run_cmd(['umount', tmp_dir])
        if res != 0:
            return (res, '%s: umount failed (%d)' % (tmp_dir, res))
    return (0, 'Success')

def sleep_some(s):
    # sleep s seconds
    ts = time.perf_counter()
    time.sleep(s)
    te = time.perf_counter()
    Global.timing['sleep'] += te - ts



"""
ISCSI classes and utilities
"""

from .util import *

class IscsiData:
    """
    Gather all the iscsi data in one place
    """
    imm_data_en = 'Yes'
    initial_r2t_en = 'No'
    hdrdgst_en = 'None,CRC32C'
    datdgst_en = 'None,CRC32C'
    first_burst = 256 * 1024
    max_burst = 16 * 1024 * 1024 - 1024
    max_recv_dlength = 128 * 1024
    max_r2t = 1
    # the target-name and IP:Port
    target = None
    ipnr = None

    def __init__(self,
            imm_data_en=imm_data_en,
            initial_r2t_en=initial_r2t_en,
            hdrdgst_en=hdrdgst_en,
            datdgst_en=datdgst_en,
            first_burst=first_burst,
            max_burst=max_burst,
            max_recv_dlength=max_recv_dlength,
            max_r2t=max_r2t):
        self.imm_data_en = imm_data_en
        self.initial_r2t_en = initial_r2t_en
        self.hdrdgst_en = hdrdgst_en
        self.datdgst_en = datdgst_en
        self.first_burst = first_burst
        self.max_burst = max_burst
        self.max_recv_dlength = max_recv_dlength
        self.max_r2t = max_r2t

    def update_cfg(self, target, ipnr):
        """
        Update the configuration -- we could do this by hacking on the
        appropriate DB file, but this is safer (and slower) by far
        """
        if Global.verbosity > 1:
            print('* ImmediateData = %s' % self.imm_data_en)
            print('* InitialR2T = %s' % self.initial_r2t_en)
            print('* HeaderDigest = %s' % self.hdrdgst_en)
            print('* DataDigest = %s' % self.datdgst_en)
            print('* FirstBurstLength = %d' % self.first_burst)
            print('* MaxBurstLength = %d' % self.max_burst)
            print('* MaxRecvDataSegmentLength = %d' % self.max_recv_dlength)
            print('* MaxOutstandingR2T = %d' % self.max_r2t)
        c = ['iscsiadm', '-m', 'node', '-T', target, '-p', ipnr, '-o', 'update']
        run_cmd(c + ['-n', 'node.session.iscsi.ImmediateData', '-v', self.imm_data_en])
        run_cmd(c + ['-n', 'node.session.iscsi.InitialR2T', '-v', self.initial_r2t_en])
        run_cmd(c + ['-n', 'node.conn[0].iscsi.HeaderDigest', '-v', self.hdrdgst_en])
        run_cmd(c + ['-n', 'node.conn[0].iscsi.DataDigest', '-v', self.datdgst_en])
        run_cmd(c + ['-n', 'node.session.iscsi.FirstBurstLength', '-v', str(self.first_burst)])
        run_cmd(c + ['-n', 'node.session.iscsi.MaxBurstLength', '-v', str(self.max_burst)])
        run_cmd(c + ['-n', 'node.conn[0].iscsi.MaxRecvDataSegmentLength', '-v', str(self.max_recv_dlength)])
        run_cmd(c + ['-n', 'node.session.iscsi.MaxOutstandingR2T', '-v', str(self.max_r2t)])



#!/usr/bin/env python3
"""
Unit tests for open-iscsi, using the unittest built-in package
"""

import sys
import unittest
import os
import time
from harness import util
#from harness import tests
#from tests import TestRegression

__version__ = '1.0'


if __name__ == '__main__':
    # do our own hackery first, to get access to verbosity, debug, etc,
    # as well as add our own command-line options
    util.setup_testProgram_overrides(__version__, 'test-open-iscsi.py')
    # now run the tests
    unittest.main(module = 'harness.tests')
